@@ -6,7 +6,7 @@
  */
 
 /*
-    Copyright 2011-2013 Thibaut Paumard, Frederic Vincent
+    Copyright 2011-2015 Thibaut Paumard, Frederic Vincent
 
     This file is part of Gyoto.
 
@@ -38,6 +38,7 @@
 #include <GyotoDefs.h>
 #include <GyotoSmartPointer.h>
 #include <GyotoConverters.h>
+#include <GyotoObject.h>
 
 namespace Gyoto{
   class Photon;
@@ -75,7 +76,7 @@ namespace Gyoto{
       (FactoryMessenger* fmp) {
       SmartPointer<T> ao = new T();
 #ifdef GYOTO_USE_XERCES
-      ao -> setParameters(fmp);
+      if (fmp) ao -> setParameters(fmp);
 #endif
       return ao;
     }
@@ -184,7 +185,10 @@ namespace Gyoto{
  *
  * See introduction in the Gyoto::Astrobj namespace.
  */
-class Gyoto::Astrobj::Generic : protected Gyoto::SmartPointee {
+class Gyoto::Astrobj::Generic
+: public Gyoto::SmartPointee,
+  public Gyoto::Object
+{
   friend class Gyoto::SmartPointer<Gyoto::Astrobj::Generic>;
 
 
@@ -200,52 +204,36 @@ class Gyoto::Astrobj::Generic : protected Gyoto::SmartPointee {
 
   /**
    * Maximum distance from the center of the coordinate system at
-   * which a photon may hit the object.  Child classes may choose to
-   * update rMax at all time or to use it to cache the value, for
-   * instance when rMax() is called. External classes (Photons in
-   * particular) must use rMax() to access this information.
-   *
-   * #rmax_set_==1 means that #rmax_ was set using rMax() or the
-   * constructor. In this case, rMax() must always return this
-   * value, not recompute it.
+   * which a photon may hit the object.  Child classes may compute a
+   * decent value for #rmax_ at any time if #rmax_ is
+   * DBL_MAX. External classes (Photons in particular) must use rMax()
+   * to access this information.
    *
    * #rmax_ is in geometrical units.
    */                                         
   double rmax_; ///< Maximum distance to the center of the coordinate system [geometrical units]
 
-  /**
-   * #rmax_set_==1 means that #rmax_ was set using rMax(double r) or the
-   * constructor. In this case, rMax() must always return this
-   * value, not recompute it.
-   *
-   * Use unsetRmax() to reset rmax_set_ to 0.
-   *
-   */                                         
-  int rmax_set_; ///< Never recompute rmax: it was externally set
-
-  /**
-   * The kind should match the name of the class, e.g. "Star" for a
-   * Gyoto::Star.
-   */
-  std::string kind_; ///< Kind of object (e.g. "Star"...)
-
-  int flag_radtransf_; ///< 1 if radiative transfer inside Astrobj, else 0
+  bool flag_radtransf_; ///< 1 if radiative transfer inside Astrobj, else 0
 
+  int radiativeq_; ///< 1 to use the new radiativeQ function (under dvp)
+  int noredshift_; ///< 1 to impose redshift factor g = 1
   // Constructors - Destructor
   // -------------------------
  public:
+  GYOTO_OBJECT;
+
   /**
-   *  #kind_ =  "Default", #rmax_ = 0., #rmax_set_ = 0.
+   *  #kind_ =  "Default", #rmax_ = DBL_MAX
    */
   Generic(); ///< Default constructor.
 
   /**
-   *  #kind_ =  "Default", #rmax_ = radmax, #rmax_set_ = 1.
+   *  #kind_ =  "Default", #rmax_ = radmax
    */
   Generic(double radmax); ///< Set rmax in constructor.
 
   /**
-   *  #kind_ =  kind, #rmax_ = 0., #rmax_set_ = 0.
+   *  #kind_ =  kind, #rmax_ = DBL_MAX
    */
   Generic(std::string kind); ///< Set kind in constructor.
 
@@ -291,14 +279,15 @@ class Gyoto::Astrobj::Generic : protected Gyoto::SmartPointee {
    *  Get maximal distance from center of coordinate system at which a
    *  Photon may hit the object.
    *  
-   *  Child classes may use the rmax_ member to cache this value.
+   *  Child classes may use the #rmax_ member to cache this value, if
+   *  its current value is DBL_MAX.
    *
-   *  It can also be set using rmax(). If rmax has been used
-   *  to set rmax_, rMax() must not recompute it.
+   *  It can also be set using rMax().
    *
    *  \return rmax_ in geometrical units
    */
   virtual double rMax(); ///< Get maximal distance from center of coordinate system
+  virtual double rMax() const; ///< Get maximal distance from center of coordinate system
 
   /**
    *  Call rMax() and convert result to unit.
@@ -306,7 +295,8 @@ class Gyoto::Astrobj::Generic : protected Gyoto::SmartPointee {
    *  \param unit string
    *  \return double rmax converted to unit
    */
-  virtual double rMax(std::string unit); ///< Get rmax_ is specified unit
+  virtual double rMax(std::string const &unit); ///< Get rmax_ is specified unit
+  virtual double rMax(std::string const &unit) const; ///< Get rmax_ is specified unit
 
   /// Get max step constraint for adaptive integration
   /**
@@ -321,7 +311,6 @@ class Gyoto::Astrobj::Generic : protected Gyoto::SmartPointee {
    *  Set maximal distance from center of coordinate system at which a
    *  Photon may hit the object.
    *  
-   *  Side effect: set #rmax_set_ to 1.
    *  \param val new #rmax_ in geometrical units.
    */
   virtual void rMax(double val); ///< Set maximal distance from center of coordinate system
@@ -333,79 +322,40 @@ class Gyoto::Astrobj::Generic : protected Gyoto::SmartPointee {
    *  \param val #rmax_ expressed in unit "unit";
    *  \param unit string...
    */
-  virtual void rMax(double val, std::string unit); ///< Set maximal distance from center of coordinate system
-
-  /**
-   * rmax() will then be free to recompute rmax_. Astrobjs
-   * which cache rmax_ may need to update it when unrmax() is
-   * called.
-   */
-  virtual void unsetRmax() ; ///< Set rmax_set_ to 0.
+  virtual void rMax(double val, std::string const &unit); ///< Set maximal distance from center of coordinate system
 
   /**
    * Set flag indicating that radiative transfer should be integrated,
    * i.e. the object is to be considered optically thin.
    * \param flag: 1 if optically thin, 0 if optically thick.
    */
-  void opticallyThin(int flag);
+  void opticallyThin(bool flag);
   ///< Set whether the object is optically thin.
   /**
-   * See opticallyThin(int flag).
+   * See opticallyThin(bool flag).
    */
-  int opticallyThin() const ;
+  bool opticallyThin() const ;
   ///< Query whether object is optically thin.
 
+  void radiativeQ(bool flag);
+  bool radiativeQ() const ;
+
+  void redshift(bool flag);
+  bool redshift() const ;
+
   /**
    * Return a Gyoto::Quantity_t suitable as input to
    * Gyoto::Scenery::setRequestedQuantities() to set de default
    * quantities to compute for this object. The default of these
    * defaults GYOTO_QUANTITY_INTENSITY.
    */
-  virtual Quantity_t getDefaultQuantities();
+  virtual Gyoto::Quantity_t getDefaultQuantities();
   ///< Which quantities to compute if know was requested
 
   //XML I/O
  public:
-  /**
-   * \brief Set parameter by name
-   *
-   * Assume MyKind is a subclass of Astrobj::Generic which has two
-   * members (a string StringMember and a double DoubleMember):
-   * \code
-   * int MyKind::setParameter(std::string name,
-   *                          std::string content,
-   *                          std::string unit) {
-   *   if      (name=="StringMember") setStringMember(content);
-   *   else if (name=="DoubleMember") setDoubleMember(atof(content.c_str()),
-   *                                                  unit);
-   *   else return Generic::setParameter(name, content, unit);
-   *   return 0;
-   * }
-   * \endcode
-   * If MyKind is not a direct subclass of Generic but is a subclass
-   * of e.g. Standard, UniformSphere of ThinDisk, it should call the
-   * corresponding setParameter() implementation instead of
-   * Generic::setParameter().
-   *
-   * \param name XML name of the parameter
-   * \param content string representation of the value
-   * \param unit string representation of the unit
-   * \return 0 if this parameter is known, 1 if it is not.
-   */
-  virtual int setParameter(std::string name,
-			   std::string content,
-			   std::string unit) ;
 
 #ifdef GYOTO_USE_XERCES
-  /**
-   * \brief Fill XML section
-   *
-   * Astrobj implementations should impement fillElement to save their
-   * parameters to XML and call the generic implementation to save
-   * generic parts such as Flag_radtrans: Generic::fillElement(fmp).
-   */
-  virtual void fillElement(FactoryMessenger *fmp) const ;
-
   /**
    * \brief Main loop in Subcontractor_t function
    *
@@ -450,7 +400,7 @@ class Gyoto::Astrobj::Generic : protected Gyoto::SmartPointee {
    * will look in it for pointers to properties which apply to its
    * kind. If a pointer to a property known to this object is present,
    * then the property is computed and store at the pointed-to
-   * adress. For instance, all objects know the "intensity"
+   * address. For instance, all objects know the "intensity"
    * property. If data->intensity != NULL, the instensity is computed
    * and stored in *data->intensity.
    *
@@ -565,6 +515,12 @@ class Gyoto::Astrobj::Generic : protected Gyoto::SmartPointee {
 			double dsem, double coord_ph[8],
 			double coord_obj[8]=NULL) const ; 
 
+  // Under development
+  virtual void radiativeQ(double Inu[], double Taunu[], 
+			  double nu_em[], size_t nbnu,
+			  double dsem, double coord_ph[8],
+			  double coord_obj[8]=NULL) const ; 
+
   /**
    * Compute the integral of emission() from &nu;<SUB>1</SUB> to
    * &nu;<SUB>2</SUB>. The default implementation is a numerical
@@ -632,7 +588,7 @@ class Gyoto::Astrobj::Generic : protected Gyoto::SmartPointee {
  *
  *  Also see Gyoto::Scenery and Gyoto::Quantity_t.
  */
-class Gyoto::Astrobj::Properties : protected Gyoto::SmartPointee {
+class Gyoto::Astrobj::Properties : public Gyoto::SmartPointee {
   friend class Gyoto::SmartPointer<Gyoto::Astrobj::Properties>;
  public:
   double *intensity; ///< GYOTO_QUANTITY_INTENSITY   : Intensity
@@ -729,6 +685,10 @@ class Gyoto::Astrobj::Properties : protected Gyoto::SmartPointee {
    */
   Gyoto::SmartPointer<Gyoto::Units::Converter> binspectrum_converter_ ;
 # endif
+
+  /// True if buffers are allocated for entire field (npix*npix)
+  bool alloc;
+
  public:
   Properties(); ///< Default constructor (everything is set to NULL);
   Properties (double*, double*); ///<< Set intensity and time pointers.
@@ -752,7 +712,7 @@ class Gyoto::Astrobj::Properties : protected Gyoto::SmartPointee {
    * All valid pointers are incremented by 1 (sizeof(double)), excepted
    * impactcoords which is incremented by 16.
    */
-  Properties operator++();
+  Properties& operator++();
 
   /**
    * \brief Increment pointers by offset
@@ -760,7 +720,9 @@ class Gyoto::Astrobj::Properties : protected Gyoto::SmartPointee {
    * All valid pointers are incremented by offset (sizeof(double)), excepted
    * impactcoords which is incremented by 16*offset.
    */
-  Properties operator+=(ptrdiff_t offset);
+  Properties& operator+=(ptrdiff_t offset);
+
+  operator Gyoto::Quantity_t () const;
 
 # ifdef HAVE_UDUNITS
   void intensityConverter(Gyoto::SmartPointer<Gyoto::Units::Converter>);
