@@ -166,9 +166,9 @@ class Gyoto::Astrobj::Complex : public Gyoto::Astrobj::Generic {
    *   cplx[0] = objB;
    * \endcode
    */
-  Gyoto::SmartPointer<Gyoto::Astrobj::Generic> operator[](size_t i) ;
+  Gyoto::SmartPointer<Gyoto::Astrobj::Generic>& operator[](size_t i) ;
   ///< Retrieve i-th element.
-  Gyoto::SmartPointer<Gyoto::Astrobj::Generic> const operator[](size_t i) const;
+  Gyoto::SmartPointer<Gyoto::Astrobj::Generic> const&operator[](size_t i) const;
   ///< Retrieve a const version of the i-th element.
   
 };
