@@ -74,6 +74,8 @@ namespace Gyoto {
 
   /* Generic */
 
+#define GYOTO_QUANTITY_NONE           0
+
   /// Intensity: I<SUB>&nu;</SUB> at Scenery::freq_obs_.
 #define GYOTO_QUANTITY_INTENSITY      1
   /// EmissionTime: Emission date.
@@ -548,6 +550,7 @@ namespace Gyoto {
 # define getDelta0          delta0
 # define setAlpha0          alpha0
 # define getAlpha0          alpha0
+# define setAnglekind       anglekind
 # define getTmin            tMin
 # define setTmin            tMin
 # define getTime            time
@@ -595,6 +598,19 @@ namespace Gyoto {
 # define setBinSpectrumConverter  binSpectrumConverter
 # define setSpectrumConverter     spectrumConverter
 # define setIntensityConverter    intensityConverter
+# define getSafetyValue           safetyValue;
+# define setSafetyValue           safetyValue;
+# define setInnerRadius           innerRadius;
+# define getInnerRadius           innerRadius;
+# define setOuterRadius           outerRadius;
+# define getOuterRadius           outerRadius;
+# define setThickness             thickness;
+# define getThickness             thickness;
+# define setDir                   dir;
+# define getDir                   dir;
+# define setBand                  band;
+# define setObserverKind          observerKind;
+# define getObserverKind          observerKind;
 //\}
 #endif
 
