@@ -29,6 +29,8 @@
 #ifndef __GyotoFactory_H_
 #define __GyotoFactory_H_
 
+#include <vector>
+
 #include "GyotoConfig.h"
 
 /// Xerces internal.
@@ -159,6 +161,13 @@ class Gyoto::Factory
 
  public:
   /// Constructor for reading a file
+  /**
+   * If filename starts with "<?xml", it is interpreted as XML data
+   * and parsed directly (with no file access). It is possible to
+   * parse a file that actually starts with those unlikely characters
+   * by escaping the first with '\'. If filename starts with '\', this
+   * single character is always stripped.
+   */
   Factory(char * filename);
 
   /// Constructor for saving (or printing) a Scenery
@@ -384,9 +393,30 @@ class Gyoto::Factory
 		    xercesc::DOMElement* pel,
 		    FactoryMessenger **child = NULL);
 
+  void setParameter(std::string name, std::vector<double> const &val,
+		    xercesc::DOMElement* pel,
+		    FactoryMessenger **child = NULL);
+
+  void setParameter(std::string name, std::vector<unsigned long> const &val,
+		    xercesc::DOMElement* pel,
+		    FactoryMessenger **child = NULL);
+
   /// Transform relative path into absolute path.
   /**
-   * \param relpath Path relative to XML file.
+   * relpath is interpreted as follows:
+   *
+   * If it starts with "/", it is interpreted as an absolute path and
+   * is returned as is.
+   *
+   * If it is prefixed with "`pwd`/", the rest of relpath is
+   * interpreted as relative to the current working directory,
+   * i.e. fullPath() tries to mimick how the shell would expand
+   * relpath into an absolute path.
+   *
+   * In any other circumstance, relpath is interpreted to relative to
+   * the directory where the XML file resides.
+   *
+   * \param relpath Path specification.
    * \return Absolute path to same file.
    */
   std::string fullPath(std::string relpath);
