@@ -34,6 +34,7 @@
 #define XERCES_INCLUDE_WCHAR_H 0
 #endif
 #include <xercesc/dom/DOMElement.hpp>
+#include <vector>
 #include <string>
 #include <GyotoDefs.h>
 #include <GyotoSmartPointer.h>
@@ -392,6 +393,11 @@ class Gyoto::FactoryMessenger {
 		    FactoryMessenger** child= NULL);
   ///< Output an array of parameters
 
+  void setParameter(std::string name, std::vector<double> const &val,
+		    FactoryMessenger** child= NULL);
+  void setParameter(std::string name, std::vector<unsigned long> const &val,
+		    FactoryMessenger** child= NULL);
+  ///< Output a vector of parameters
 
   /**
    * For instance Spectrometer::fillElement() sets its "kind"
@@ -456,6 +462,31 @@ class Gyoto::FactoryMessenger {
   FactoryMessenger* makeChild(std::string name);
   ///< Create child FactoryMessenger
 
+  /**
+   * \brief Parse string into array
+   *
+   * Parse at most max_tokens tokens from string src into
+   * pre-allocated array dst. Returns the number of tokens actually
+   * found (interpreted using atof). dst must be at least of size
+   * max_tokens.
+   */
+  static size_t parseArray(std::string src, double dst[], size_t max_tokens);
+
+  /**
+   * \brief Parse string into array
+   *
+   * Parse tokens from string src, returns them into a
+   * std::vector<double>.
+   */
+  static std::vector<double> parseArray(std::string src);
+
+  /**
+   * \brief Parse string into array
+   *
+   * Parse tokens from string src, returns them into a
+   * std::vector<unsigned long>.
+   */
+  static std::vector<unsigned long> parseArrayULong(std::string src);
 };
 
 #endif
