@@ -69,6 +69,7 @@ class Gyoto::Metric::KerrBL : public Metric::Generic {
   // Constructors - Destructor
   // -------------------------
  public: 
+  GYOTO_OBJECT;
   KerrBL(); ///< Default constructor
   virtual KerrBL * clone () const ;
 
@@ -86,10 +87,14 @@ class Gyoto::Metric::KerrBL : public Metric::Generic {
   void genericIntegrator(bool);
   bool genericIntegrator() const ;
 
-  double getRms() const; ///< Returns prograde marginally stable orbit
+  virtual double getRms() const; 
 
-  double getRmb() const; ///< Returns prograde marginally bound orbit
+  virtual double getRmb() const; 
+
+  virtual double getSpecificAngularMomentum(double rr) const;
   
+  virtual double getPotential(double pos[4], double l_cst) const;
+
   void gmunu(double g[4][4], const double * pos) const ;
   double gmunu(const double * const x, int mu, int nu) const ;
 
@@ -121,11 +126,6 @@ class Gyoto::Metric::KerrBL : public Metric::Generic {
   void MakeMomentum(const double coordin[8], const double cst[5], double coordout[8]) const;
   ///< Transforms from Boyer-Lindquist coordinates [t,r,th,phi,tdot,rdot,thdot,phidot] to [t,r,th,phi,pt,pr,pth,pphi] where pt,pr... are generalized momenta.
 
-  virtual void setParameter(std::string, std::string, std::string);
-#ifdef GYOTO_USE_XERCES
-  virtual void fillElement(FactoryMessenger *fmp); ///< called from Factory
-#endif
-
  protected:
 
   // outside the API
@@ -135,14 +135,15 @@ class Gyoto::Metric::KerrBL : public Metric::Generic {
    */
   int myrk4(Worldline * line, const double coordin[8], double h, double res[8]) const; //external-use RK4
   
- private:
+ public:
   int myrk4(const double coor[8], const double cst[5], double h, double res[8]) const;///< Internal-use RK4 proxy
-  int myrk4_adaptive(Gyoto::Worldline* line, const double coor[8], double lastnorm, double normref, double coor1[8], double h0, double& h1, double h1max) const; ///< Internal-use adaptive RK4 proxy
+  int myrk4_adaptive(Gyoto::Worldline* line, const double coor[8], double lastnorm, double normref, double coor1[8], double h0, double& h1, double h1max=GYOTO_DEFAULT_DELTA_MAX) const; ///< Internal-use adaptive RK4 proxy
   /**
    * \brief Ensure conservation of the constants of motion
    *
    * Tweak thetadot if necessary.
    */
+ private:
   int CheckCons(const double coor_init[8], const double cst[5], double coor_fin[8]) const;
 
   /**
@@ -152,9 +153,11 @@ class Gyoto::Metric::KerrBL : public Metric::Generic {
    */
   void Normalize4v(double coord[8], const double part_mass) const;
 
+ public:
   /** F function such as dy/dtau=F(y,cst)
    */
   using Metric::Generic::diff;
+ private:
   /** 
    * \brief Used in RK4 proxies.
    */
@@ -167,6 +170,10 @@ class Gyoto::Metric::KerrBL : public Metric::Generic {
   void setParticleProperties(Worldline* line, const double* coord) const;
   virtual int isStopCondition(double const * const coord) const;
   
+  void observerTetrad(std::string const obskind,
+		      double const pos[4], double fourvel[4],
+		      double screen1[4], double screen2[4], 
+		      double screen3[4]) const;
 };
 
 #endif
