@@ -36,6 +36,7 @@
 #include <string>
 
 #include <GyotoSmartPointer.h>
+#include <GyotoObject.h>
 #include <GyotoAstrobj.h>
 #include <GyotoRegister.h>
 #include <GyotoHooks.h>
@@ -69,7 +70,7 @@ namespace Gyoto {
       (FactoryMessenger* fmp) {
       SmartPointer<T> gg = new T();
 #ifdef GYOTO_USE_XERCES
-      gg -> setParameters(fmp);
+      if (fmp) gg -> setParameters(fmp);
 #endif
       return gg;
     }
@@ -145,13 +146,13 @@ namespace Gyoto {
  *
  */
 class Gyoto::Metric::Generic
-: protected Gyoto::SmartPointee,
+: public Gyoto::SmartPointee,
+  public Gyoto::Object,
   public Gyoto::Hook::Teller
 {
   friend class Gyoto::SmartPointer<Gyoto::Metric::Generic>;
 
  private:
-  std::string kind_; ///< Metric kind name (e.g. "KerrBL")
   double mass_;     ///< Mass yielding geometrical unit (in kg).
   int coordkind_; ///< Kind of coordinates (cartesian-like, spherical-like, unspecified)
 
@@ -194,13 +195,15 @@ class Gyoto::Metric::Generic
 
 
  public:
+  GYOTO_OBJECT;
+
   const std::string kind() const; ///< Get kind_
   int getRefCount();
   
   // Constructors - Destructor
   // -------------------------
   Generic(const int coordkind, const std::string &name); ///< Constructor setting Generic::coordkind_ and kind_
-
+  Generic(Generic const &o); ///< Copy constructor
   virtual ~Generic() ;                        ///< Destructor
   
   // Mutators / assignment
@@ -225,6 +228,34 @@ class Gyoto::Metric::Generic
   double unitLength() const ; ///< M * G / c^2, M is in kg, unitLength in meters
   double unitLength(const std::string &unit) const ; ///< unitLength expressed in specified unit
 
+  /**
+   * Returns the marginally bound radius
+   * Should be implemented in derived classes if useful
+   * If called on the base class, returns an error
+   */
+  virtual double getRmb() const;
+
+  /**
+   * Returns the marginally stable (ISCO) radius 
+   * Should be implemented in derived classes if useful
+   * If called on the base class, returns an error
+   */
+  virtual double getRms() const;
+
+  /**
+   * Returns the specific angular momentum l=-u_phi/u_t
+   * Should be implemented in derived classes if useful
+   * If called on the base class, returns an error
+   */
+  virtual double getSpecificAngularMomentum(double rr) const;
+
+  /**
+   * Returns potential W=-ln(|u_t|) for a cst specific angular momentum l_cst
+   * Should be implemented in derived classes if useful
+   * If called on the base class, returns an error
+   */
+  virtual double getPotential(double pos[4], double l_cst) const;
+
   /**
    * Get delta_min_
    */
@@ -339,70 +370,21 @@ class Gyoto::Metric::Generic
  
 
   /**
-   * \brief Set parameter by name
-   *
-   * Assume MyKind is a subclass of Metric::Generic which has two
-   * members (a string StringMember and a double DoubleMember):
-\code
-int MyKind::setParameter(std::string name, std::string content, std::string unit) {
- if      (name=="StringMember") setStringMember(content);
- else if (name=="DoubleMember") setDoubleMemeber(atof(content.c_str()), unit);
- else return Generic::setParameter(name, content, unit);
- return 0;
-}
-\endcode
-   * If MyKind is not a direct subclass of Generic, it should call the
-   * corresponding setParameter() implementation instead of
-   * Generic::setParameter().
-   *
-   * \param name XML name of the parameter
-   * \param content string representation of the value
-   * \param unit string representation of the unit
-   * \return 0 if this parameter is known, 1 if it is not.
+   * \brief Computes the orthonormal local tetrad of the observer
+   * 
+   * \param obskind  input: kind of observer (eg: "ZAMO","KeplerianObserver"...)
+   * \param pos      input: position,
+   * \param fourvel output: observer 4-velocity (norm -1)
+   * \param screen1 output: first vector in the screen plane
+   * \param screen2 output: second vector in the screen plane
+   * \param screen3 output: vector normal to the screen
    */
-  virtual void setParameter(std::string name,
-			    std::string content,
-			    std::string unit);
+  virtual void observerTetrad(std::string const obskind,
+			      double const pos[4], double fourvel[4],
+			      double screen1[4], double screen2[4],
+			      double screen3[4]) const ;
 
   // Outputs
-#ifdef GYOTO_USE_XERCES
-
-  /**
-   * \brief Main loop in Subcontractor_t function
-   *
-   * The Subcontractor_t function for each Metric kind should look
-   * somewhat like this (templated as
-   * Gyoto::Metric::Subcontractor<MyKind>):
-\code
-SmartPointer<Metric::Generic>
-Gyoto::Metric::MyKind::Subcontractor(FactoryMessenger* fmp) {
-  SmartPointer<MyKind> gg = new MyKind();
-  gg -> setParameters(fmp);
-  return gg;
-}
-\endcode
-   *
-   * Each metric kind should implement setParameter(string name,
-   * string content, string unit) to interpret the individual XML
-   * elements. setParameters() can be overloaded in case the specific
-   * Metric class needs low level access to the FactoryMessenger. See
-   * Gyoto::Astrobj::UniformSphere::setParameters().
-   */
-  virtual void setParameters(Gyoto::FactoryMessenger *fmp) ;
-
-  /**
-   * Metrics implementations should impement fillElement to save their
-   * parameters to XML and call the Metric::fillElement(fmp) for the
-   * shared properties
-   */
-
-  virtual void fillElement(FactoryMessenger *fmp) ; ///< called from Factory
-
-  /**
-   * \brief Process generic XML parameters 
-   */
-  void processGenericParameters(Gyoto::FactoryMessenger *fmp) ;
-#endif
 
   /**
    * \brief Metric coefficients
