@@ -6,7 +6,7 @@
  */
 
 /*
- *   Copyright (c) 2012 Frederic Vincent
+ *   Copyright (c) 2012-2015 Frederic Vincent, Thibaut Paumard
  *
  *
  */
@@ -22,10 +22,13 @@ namespace Gyoto {
   class FactoryMessenger;
 }
 
-class Scalar;
-class Vector;
-class Sym_tensor;
-class Valeur;
+// Forward declarations of Lorene classes
+namespace Lorene {
+  class Scalar;
+  class Vector;
+  class Sym_tensor;
+  class Valeur;
+}
 
 #include <GyotoMetric.h>
 #include <GyotoWorldline.h>
@@ -49,7 +52,8 @@ class Gyoto::Metric::NumericalMetricLorene
 
  private:
   char* filename_; ///< Lorene .d data file(s) path
-  char* mapkind_; ///< Kind of Lorene mapping Map_af or Map_et
+  bool mapet_; ///< Kind of Lorene mapping: 'false' for Map_af, 'true' for Map_et
+  bool bosonstarcircular_; ///< 1 to implement the circular velocity of a boson star
   int has_surface_; ///< 1 if the metric source has a surface
   int specify_marginalorbits_; ///< 1 if marginal orbits are specified in file
   double horizon_; ///< Value of horizon (or any innermost limit)
@@ -57,23 +61,25 @@ class Gyoto::Metric::NumericalMetricLorene
   double h0_refine_; ///< Imposed integration step for refined integration
   int refine_; ///< 1 if refined integration needed
   double initial_time_; ///< Time at which (first) metric is given
-  Scalar** lapse_tab_;
-  Vector** shift_tab_;
-  Sym_tensor** gamcov_tab_;
-  Sym_tensor** gamcon_tab_;
-  Sym_tensor** kij_tab_;
+  Lorene::Scalar** lapse_tab_;
+  Lorene::Vector** shift_tab_;
+  Lorene::Sym_tensor** gamcov_tab_;
+  Lorene::Sym_tensor** gamcon_tab_;
+  Lorene::Sym_tensor** kij_tab_;
   double* times_; ///< Coordinate times at which metrics are given
   int nb_times_; ///< Nb of time slices
-  Valeur** nssurf_tab_; ///< Metric source (e.g. star) surface (if any)
-  Vector** vsurf_tab_; ///< 4-velocity at surface (if any)
-  Scalar** lorentz_tab_; ///< Lorentz factor at surface (if any)
-  Valeur** hor_tab_; ///< Apparent horizon (if any)
+  Lorene::Valeur** nssurf_tab_; ///< Metric source (e.g. star) surface (if any)
+  Lorene::Vector** vsurf_tab_; ///< 4-velocity at surface (if any)
+  Lorene::Scalar** lorentz_tab_; ///< Lorentz factor at surface (if any)
+  Lorene::Valeur** hor_tab_; ///< Apparent horizon (if any)
   double risco_; ///< ISCO coordinate radius
+  double rico_; ///< Innermost circular orbit coordinate radius
   double rmb_; ///< Marginally bound orbit coordinate radius
 
   void free(); ///< deallocate memory
 
  public:
+  GYOTO_OBJECT;
   NumericalMetricLorene(); ///< Constructor
   NumericalMetricLorene(const NumericalMetricLorene&); ///< Copy constructor
   virtual NumericalMetricLorene* clone() const ;
@@ -83,24 +89,46 @@ class Gyoto::Metric::NumericalMetricLorene
    * Access functions to get or set private attributes
    */
   virtual void setMetricSource();
-  char const * getFileName() const;
-  Vector** getShift_tab() const;
-  Scalar** getLapse_tab() const;
-  Sym_tensor** getGamcon_tab() const;
+
+  void        directory(std::string const &dir) ;
+  std::string directory() const ;
+  double initialTime() const ;
+  void   initialTime(double t0);
+  double horizon() const ;
+  void   horizon(double t0);
+  double rico() const ;
+  void   rico(double r0);
+  bool hasSurface() const;
+  void hasSurface(bool s);
+  bool bosonstarcircular() const;
+  void bosonstarcircular(bool);
+  bool specifyMarginalOrbits() const;
+  void specifyMarginalOrbits(bool s);
+  bool mapEt() const;
+  void mapEt(bool s);
+  std::vector<double> refineIntegStep() const;
+  void refineIntegStep(std::vector<double> const&);
+
+  Lorene::Vector** getShift_tab() const;
+  Lorene::Scalar** getLapse_tab() const;
+  Lorene::Sym_tensor** getGamcon_tab() const;
   double* getTimes() const;
   int getNbtimes() const;
-  Valeur** getNssurf_tab() const;
-  Vector** getVsurf_tab() const;
-  Scalar** getLorentz_tab() const;
-  Valeur** getHor_tab() const;
-  double getRisco() const;
+  Lorene::Valeur** getNssurf_tab() const;
+  Lorene::Vector** getVsurf_tab() const;
+  Lorene::Scalar** getLorentz_tab() const;
+  Lorene::Valeur** getHor_tab() const;
+  double getRms() const;
   double getRmb() const;
-  void setLapse_tab(Scalar* lapse, int ii);
-  void setShift_tab(Vector* shift, int ii);
-  void setGamcov_tab(Sym_tensor* gamcov, int ii);
-  void setGamcon_tab(Sym_tensor* gamcon, int ii);
-  void setKij_tab(Sym_tensor* kij, int ii);
+  void setLapse_tab(Lorene::Scalar* lapse, int ii);
+  void setShift_tab(Lorene::Vector* shift, int ii);
+  void setGamcov_tab(Lorene::Sym_tensor* gamcov, int ii);
+  void setGamcon_tab(Lorene::Sym_tensor* gamcon, int ii);
+  void setKij_tab(Lorene::Sym_tensor* kij, int ii);
   void setTimes(double time,int ii);
+
+  virtual double getSpecificAngularMomentum(double rr) const;
+  virtual double getPotential(double pos[4], double l_cst) const;  
   
   /**
    * Runge-Kutta integrator at order 4
@@ -144,6 +172,15 @@ class Gyoto::Metric::NumericalMetricLorene
   
   double christoffel(const double coord[8], const int alpha, const int mu, 
 		     const int nu) const ;
+  double christoffel(const double coord[8], 
+		     const int alpha, 
+		     const int mu, const int nu,
+		     const int indice_time) const;
+  virtual int christoffel(double dst[4][4][4], 
+			  const double * coord) const;
+  int christoffel(double dst[4][4][4], 
+		  const double * coord,
+		  const int indice_time) const;
   /**
    * 3-Christoffels
    */
@@ -168,15 +205,34 @@ class Gyoto::Metric::NumericalMetricLorene
   /**
    * F function such as d(coord)/d(tau)=F(coord)
    */
-  using Generic::diff;
+  //using Generic::diff;
+  virtual int diff(const double coord[8], double res[8]) const;
   int diff(double tt, const double y[7], double res[7]) const ;
   virtual int diff(const double y[7], double res[7], int indice_time) const ;
 
-  virtual void setParameter(std::string, std::string, std::string);
-#ifdef GYOTO_USE_XERCES
-  virtual void fillElement(FactoryMessenger *fmp); /// < called from Factory
-  virtual void setParameters(FactoryMessenger *fmp);
-#endif
+  /**
+   * \brief Yield circular velocity at a given position
+   * 
+   * Give the velocity of a massive particle in circular orbit at the
+   * given position projected onto the equatorial plane. Such a
+   * velocity may not exist everywhere (or anywhere) for a given
+   * metric. This method is intended to be used by Astrobj classes
+   * such as Torus or ThinDisk.
+   *
+   * This circular velocity should be implemented for all specific
+   * numerical metric used.
+   *
+   * If bosonstarcircular_ is set to true, this method returns the
+   * boson star circular velocity.
+   *
+   * \param coor input: position,
+   * \param vel output: velocity,
+   * \param dir 1 for corotating, -1 for counterrotating.
+   */
+  void circularVelocity(double const * coor, double* vel,
+			double dir) const ;
+  void circularVelocity(double const * coor, double* vel,
+			double dir, int indice_time) const ;
 
 };
 
