@@ -12,7 +12,7 @@
  */
 
 /*
-  Copyright 2011 Frederic Vincent, Thibaut Paumard
+  Copyright 2011-2015 Frederic Vincent, Thibaut Paumard
   
   This file is part of Gyoto.
   
@@ -75,14 +75,15 @@ class Gyoto::Astrobj::PageThorneDisk
   double x3_; ///< Value cached for bolometricEmission()
   int blackbody_; ///< Flag for computing BB flux (for spectra)
   double mdot_; ///< accretion rate (for BB spectrum computation)
-  int uniflux_; ///< Flag for uniform flux = 1
+  bool uniflux_; ///< Flag for uniform flux = 1
   SmartPointer<Spectrum::BlackBody> spectrumBB_; ///< disk black body
   ///< emission law
   
   // Constructors - Destructor
   // -------------------------
  public:
-  
+  GYOTO_OBJECT;
+
   PageThorneDisk(); ///< Standard constructor
   
   PageThorneDisk(const PageThorneDisk& ) ;///< Copy constructor
@@ -97,6 +98,14 @@ class Gyoto::Astrobj::PageThorneDisk
   virtual void metric(SmartPointer<Metric::Generic>);
   ///< Set metric, checking that it is either KerrBL or KerrKS
 
+  /// Set #mdot_ to v, and #blackbody_ to true
+  void BlackbodyMdot(double v);
+  double BlackbodyMdot() const;
+  void blackBody(bool t) ;
+  bool blackBody() const ;
+  void uniFlux(bool t) ;
+  bool uniFlux() const ;
+
  private:
   virtual void updateSpin() ;
   ///< Get spin from metric, which must be KerrBL or KerrKS
@@ -108,7 +117,7 @@ class Gyoto::Astrobj::PageThorneDisk
    * Throws a Gyoto::Error
    */
   virtual double emission(double nu_em, double dsem,
-			  double c_ph[8], double c_obj[8]) const;
+			  double c_ph[8], double c_obj[8]=NULL) const;
 
   /**
    * \brief Bolometric emission
@@ -128,11 +137,7 @@ virtual double bolometricEmission(double nuem, double dsem,
                                    double* coord_obj_hit, double dt,
                                    Astrobj::Properties* data) const;
 
-  int setParameter(std::string name,
-		   std::string content,
-		   std::string unit);
-
-  Quantity_t getDefaultQuantities();
+  Gyoto::Quantity_t getDefaultQuantities();
 
   // Hook::Listener API //
  public:
@@ -145,11 +150,6 @@ virtual double bolometricEmission(double nuem, double dsem,
    */
   virtual void tell(Gyoto::Hook::Teller *msg);
 
- public:
-#ifdef GYOTO_USE_XERCES
-  virtual void fillElement(FactoryMessenger *fmp) const ; ///< called from Factory
-#endif
-
 };
 
 #endif
