@@ -7,7 +7,7 @@
  */
 
 /*
-    Copyright 2011 Frederic Vincent, Thibaut Paumard
+    Copyright 2011-2015 Frederic Vincent, Thibaut Paumard
 
     This file is part of Gyoto.
 
@@ -69,6 +69,8 @@ class Gyoto::Astrobj::PatternDiskBB : public Astrobj::PatternDisk {
   // Constructors - Destructor
   // -------------------------
  public:
+  GYOTO_OBJECT;
+
   PatternDiskBB(); ///< Standard constructor
   
   PatternDiskBB(const PatternDiskBB& ) ;///< Copy constructor
@@ -79,13 +81,15 @@ class Gyoto::Astrobj::PatternDiskBB : public Astrobj::PatternDisk {
   // Accessors
   // ---------
  public:
-
-  int setParameter(std::string name, std::string content, std::string unit);
+  bool spectralEmission() const;
+  void spectralEmission(bool t);
+  double risco() const;
+  void risco(double r);
 
  public:
   using PatternDisk::emission;
   double emission(double nu_em, double dsem,
-		  double c_ph[8], double c_obj[8]) const;
+		  double c_ph[8], double c_obj[8]=NULL) const;
   
   double const * getVelocity() const ;
   void getVelocity(double const pos[4], double vel[4])  ;
@@ -93,11 +97,6 @@ class Gyoto::Astrobj::PatternDiskBB : public Astrobj::PatternDisk {
   using Generic::metric;
   void metric(SmartPointer<Metric::Generic> gg); ///< Insures metric is KerrBL or alike
 
- public:
-#ifdef GYOTO_USE_XERCES
-  void fillElement(FactoryMessenger *fmp) const ;
-#endif
-
 };
 
 #endif
