@@ -7,7 +7,7 @@
  */
 
 /*
-    Copyright 2011 Frederic Vincent, Thibaut Paumard
+    Copyright 2011, 2015 Frederic Vincent, Thibaut Paumard
 
     This file is part of Gyoto.
 
@@ -39,6 +39,7 @@ namespace Gyoto{
 #include <GyotoMetric.h>
 #include <GyotoScreen.h>
 #include <GyotoWorldline.h>
+#include <GyotoObject.h>
 
 #include <float.h>
 
@@ -48,7 +49,11 @@ namespace Gyoto{
  *
  * This is the central object for ray-tracing. 
  */
-class Gyoto::Photon : public Gyoto::Worldline, protected Gyoto::SmartPointee {
+class Gyoto::Photon
+: public Gyoto::Worldline,
+  public Gyoto::SmartPointee,
+  public Gyoto::Object
+{
   friend class Gyoto::SmartPointer<Gyoto::Photon>;
   // Data : 
   // -----
@@ -90,6 +95,8 @@ class Gyoto::Photon : public Gyoto::Worldline, protected Gyoto::SmartPointee {
   // -------------------------
 
  public:
+  GYOTO_OBJECT;
+  GYOTO_WORLDLINE;
   virtual std::string className() const ; ///< "Photon"
   virtual std::string className_l() const ; ///< "photon"
 
@@ -120,6 +127,11 @@ class Gyoto::Photon : public Gyoto::Worldline, protected Gyoto::SmartPointee {
   /// Get Photon::object_
   SmartPointer<Astrobj::Generic> astrobj() const ;
 
+  using Worldline::metric;
+  // Set Worldline::metric_ as well as the Astrobj metric
+  void metric(SmartPointer<Metric::Generic>);
+
+
   /// Set Photon::spectro_
   void spectrometer(SmartPointer<Spectrometer::Generic> spr);
   /// Get Photon::spectro_
@@ -216,10 +228,7 @@ class Gyoto::Photon : public Gyoto::Worldline, protected Gyoto::SmartPointee {
 		 double tinside, double &toutside) ;
 
 #ifdef GYOTO_USE_XERCES
- public:
-  /// Write XML description
-  void fillElement(FactoryMessenger *fmp) const;
-  /// Instanciate Photon from XML description
+  virtual void setParameters(FactoryMessenger *fmp) ;
   static SmartPointer<Photon> Subcontractor(Gyoto::FactoryMessenger*);
 #endif
 
