@@ -10,7 +10,7 @@
  */
 
 /*
-    Copyright (c) 2012 Frederic Vincent, Odele Straub, Thibaut Paumard
+    Copyright (c) 2012-2015 Frederic Vincent, Odele Straub, Thibaut Paumard
 
     This file is part of Gyoto.
 
@@ -36,12 +36,10 @@ namespace Gyoto{
  class FactoryMessenger;
 }
 
-//#include <GyotoMetric.h>
-#include <GyotoKerrBL.h>
 #include <GyotoStandardAstrobj.h>
 #include <GyotoFunctors.h>
 #include <GyotoHooks.h>
-//#include <GyotoPolishDoughnutCst.h>
+#include <GyotoBlackBodySpectrum.h>
 
 /**
  * \brief A toroïdal accretion structure
@@ -61,21 +59,14 @@ class Gyoto::Astrobj::PolishDoughnut
  // Data : 
  // -----
 protected:
-  /**
-   * \brief KerrBL metric
-   *
-   * This Astrobj::Generic subclass only works (so far?) in KerrBL
-   * metric.  We store a copy of the Astrobj::Generic::gg_
-   * SmartPointer appropriately cast. The two pointers point to the
-   * same instance.
-   */
- SmartPointer<Gyoto::Metric::KerrBL> gg_; 
+ SmartPointer<Spectrum::BlackBody> spectrumBB_;
  double l0_; ///< Angular momentum. Tied to PolishDoughnut::lambda_.
  double lambda_; ///< Adimentionned angular momentum
  double W_surface_; ///< Potential surface value. Tied to PolishDoughnut::lambda_.
  double W_centre_; ///< Potential central value. Tied to PolishDoughnut::lambda_.
  double r_cusp_; ///< Cusp radius in geometrical units. Tied to PolishDoughnut::lambda_.
  double r_centre_; ///< Central radius in geometrical units. Tied to PolishDoughnut::lambda_.
+ double r_torusouter_ ; ///< Torus outer coordinate radius. Tied to PolishDoughnut::lambda_.
  double DeltaWm1_; ///< 1./(W_centre_ - W_surface_);
  double central_density_; ///< Central density in kg/L (same as g cm^-3)
  /*
@@ -91,11 +82,33 @@ protected:
  bool komissarov_; ///< 1 if Komissarov model is integrated
  bool angle_averaged_; ///< 1 if Komissarov model should be angle averaged
 
+ /// fraction of thermal energy in non-thermal electrons
+ /**
+  * Obsiously, 0 means no non-thermal electrons at all. In this case,
+  * the (trivial) non-thermal computations are skipped. Ther is thus
+  * non need for an additional "nonthermal_" flag.
+  */
+ double deltaPL_;
+ double expoPL_; ///< exponent of the non-thermal powerlaw = -expoPL_
+
+ bool adaf_; ///< true to switch to an ADAF model rather tha Polish doughnut
+ double ADAFtemperature_; ///< ADAF central temperature
+ double ADAFdensity_; ///< ADAF central density
+
+ bool changecusp_; ///< true to apply the fishy rcusp_ change (to be changed)
+ bool rochelobefilling_; ///< true if torus filling its Roche lobe
+ bool defangmomrinner_; ///< true if torus defined from l0 and rin
+ double rintorus_; ///< Inner radius of the doughnut
+
  // Constructors - Destructor
  // -------------------------
 public:
+ GYOTO_OBJECT; // This object has Properties
+#ifdef GYOTO_USE_XERCES
+ // We need to filter some properties when writing XML
+ void fillProperty(Gyoto::FactoryMessenger *fmp, Property const &p) const;
+#endif
  PolishDoughnut() ; ///< Default constructor
- //PolishDoughnut(SmartPointer<Metric::KerrBL> gg, double lambda) ; ///< Standard constructor
  PolishDoughnut(const PolishDoughnut& ) ;                ///< Copy constructor
  virtual  PolishDoughnut* clone() const;
  virtual ~PolishDoughnut() ;                        ///< Destructor
@@ -117,9 +130,9 @@ public:
  void   lambda(double lambda); ///< Set PolishDoughnut::lambda_
 
  double centralDensity() const; ///< Get PolishDoughnut::central_density_
- double centralDensity(std::string unit) const; ///< Get PolishDoughnut::central_density_ in specified unit
+ double centralDensity(std::string const &unit) const; ///< Get PolishDoughnut::central_density_ in specified unit
  void   centralDensity(double density); ///< Set PolishDoughnut::central_density_
- void   centralDensity(double density, std::string unit); ///< Set PolishDoughnut::central_density_ in specified unit
+ void   centralDensity(double density, std::string const &unit); ///< Set PolishDoughnut::central_density_ in specified unit
 
  double centralTempOverVirial() const; ///< Get PolishDoughnut::centraltemp_over_virial_
  void   centralTempOverVirial(double val); ///< Set PolishDoughnut::centraltemp_over_virial_
@@ -130,8 +143,30 @@ public:
  void   spectralOversampling(size_t); ///< Set PolishDoughnut::spectral_oversampling_
  size_t spectralOversampling() const ; ///< Get PolishDoughnut::spectral_oversampling_
 
+ bool changeCusp() const; ///< Get PolishDoughnut::komissarov_
+ void changeCusp(bool t); ///< Set PolishDoughnut::komissarov_
  bool komissarov() const; ///< Get PolishDoughnut::komissarov_
  void komissarov(bool komis); ///< Set PolishDoughnut::komissarov_
+ bool angleAveraged() const; ///< Get PolishDoughnut::angle_averaged_
+
+ /**
+  * if komis, also sets komissarov_ to true
+  */
+ void angleAveraged(bool komis); ///< Set PolishDoughnut::angle_averaged_
+
+ void nonThermalDeltaExpo(std::vector<double> const &v);
+ std::vector<double> nonThermalDeltaExpo() const;
+ void angmomrinner(std::vector<double> const &v);
+ std::vector<double> angmomrinner() const;
+ void adafparams(std::vector<double> const &v);
+ std::vector<double> adafparams() const;
+ void adaf(bool t);
+ bool adaf() const;
+ void setParameter(Gyoto::Property const &p,
+		   std::string const & name,
+		   std::string const & content,
+		   std::string const & unit);
+
 
  // Read only members, depend on lambda
  double getWsurface() const; ///< Get PolishDoughnut::W_surface_
@@ -139,7 +174,7 @@ public:
  double getRcusp() const; ///< Get PolishDoughnut::r_cusp_
  double getRcentre() const; ///< Get PolishDoughnut::r_centre_
 
- virtual Gyoto::SmartPointer<Gyoto::Metric::Generic> metric() const;
+ using Standard::metric;
  virtual void metric(Gyoto::SmartPointer<Gyoto::Metric::Generic>);
 
  // ASTROBJ API
@@ -149,13 +184,6 @@ public:
 
  virtual double operator()(double const coord[4]) ;
 
- virtual int setParameter(std::string name,
-			  std::string content,
-			  std::string unit) ;
-#ifdef GYOTO_USE_XERCES
-  virtual void fillElement(FactoryMessenger *fmp) const ;
-#endif
-
   // ASTROBJ processHitQuantities API
   // --------------------------------
 protected:
@@ -187,9 +215,10 @@ protected:
 			double dsem, double coord_ph[8],
 			double coord_obj[8]=NULL) const ;
 
-  void emission_komissarov(double Inu[], double nu_em[], size_t nbnu,
-			double dsem, double coord_ph[8],
-			double coord_obj[8]=NULL) const ;
+  virtual void radiativeQ(double Inu[], double Taunu[], 
+			  double nu_em[], size_t nbnu,
+			  double dsem, double coord_ph[8],
+			  double coord_obj[8]=NULL) const ;
 
   double emissionBrems(double nu_em, double nu_crit, 
 		       double numax, double T_electron,
@@ -206,18 +235,38 @@ protected:
 		       double alpha1, double alpha2,
 		       double alpha3, double preff,
 		       int comptonorder) const;
-
   double emissionSynchro_komissarov_direction(double Theta_elec, 
 					      double number_density,
 					      double nuem,
 					      double nuc, 
 					      double theta
-					      ) const;
+					      )  const;
   double emissionSynchro_komissarov_averaged(double Theta_elec, 
 					     double number_density,
 					     double nuem,
 					     double nuc 
 					     ) const;
+  double emissionSynchro_komissarov_averaged_integ(double Theta_elec, 
+						   double number_density,
+						   double nuem,
+						   double nuc 
+						   ) const;
+  double emissionSynchro_komissarov_PL_direction(
+						 double number_density_PL,
+						 double nuem, double nuc,
+						 double theta_mag) const;
+  double emissionSynchro_komissarov_PL_averaged(
+						 double number_density_PL,
+						 double nuem, double nuc
+						) const;
+  double absorptionSynchro_komissarov_PL_direction(
+						   double number_density_PL,
+						   double nuem, double nuc,
+						   double theta_mag) const ;
+  double absorptionSynchro_komissarov_PL_averaged(
+						  double number_density_PL,
+						  double nuem, double nuc
+						  ) const;
   ///< Synchrotron proxy for emission()
   double transmission(double nuem, double dsem, double coord_ph[8]) const ;
   double BBapprox(double nuem, double Te) const; ///< Approximated Black-Body function
@@ -274,10 +323,24 @@ protected:
     * \endcode
     */
     double const * par;
+    const PolishDoughnut * papa;
     virtual double operator() (double) const;
   };
  intersection_t intersection; ///< double intersection(double) Functor
 
+/**
+  * \class Gyoto::Astrobj::PolishDoughnut::outerradius_t
+  * \brief double outerradius(double) Functor class
+  *
+  * To find the outer doughnut radius.
+  * This class is as a local variable in PolishDoughnut::emission()
+  */
+  class outerradius_t : public Gyoto::Functor::Double_Double_const {
+  public:
+    const PolishDoughnut * papa;
+    virtual double operator() (double) const;
+  };
+
  public:
  static double bessi0(double xx);///< Modified Bessel function I<SUB>0</SUB>
  static double bessi1(double xx);///< Modified Bessel function I<SUB>1</SUB>
