@@ -31,7 +31,9 @@
 #include <iostream>
 #include <fstream>
 
-class Star_rot;
+namespace Lorene{
+  class Star_rot;
+}
 
 namespace Gyoto {
   namespace Metric { class RotStar3_1; }
@@ -54,11 +56,11 @@ class Gyoto::Metric::RotStar3_1 : public Gyoto::Metric::Generic {
 
  private:
   char* filename_; ///< Lorene output file name
-  Star_rot * star_; ///< Pointer to underlying Lorene Star_rot instance 
+  Lorene::Star_rot * star_; ///< Pointer to underlying Lorene Star_rot instance 
   int integ_kind_;///< 1 if RotStar3_1::myrk4(), 0 if Metric::myrk4()
 
  public:
-
+  GYOTO_OBJECT;
   RotStar3_1(); ///< Constructor
   RotStar3_1(const RotStar3_1& ) ;                ///< Copy constructor
   virtual ~RotStar3_1() ;        ///< Destructor
@@ -68,8 +70,13 @@ class Gyoto::Metric::RotStar3_1 : public Gyoto::Metric::Generic {
   void fileName(char const *); ///< Set filename_
   char const * fileName() const; ///< Get filename_
 
+  void file(std::string const &); ///< Set filename_
+  std::string file() const; ///< Get filename_
+
   void integKind(int); ///< Set integ_kind_
   int integKind() const ; ///< Get integ_kind_
+  void genericIntegrator(bool); ///< Set !integ_kind_
+  bool genericIntegrator() const ;///< Get !integ_kind_
 
   using Metric::Generic::myrk4;
 
@@ -124,11 +131,7 @@ class Gyoto::Metric::RotStar3_1 : public Gyoto::Metric::Generic {
   double ScalarProd(const double pos[4],
 		    const double u1[4], const double u2[4]) const ;
 
-  virtual void setParameter(std::string, std::string, std::string);
-#ifdef GYOTO_USE_XERCES
-  virtual void fillElement(FactoryMessenger *fmp); ///< called from Factory
-  virtual void setParameters(Gyoto::FactoryMessenger *fmp) ;
-#endif
+  virtual int setParameter(std::string, std::string, std::string);
 
 };
 
