@@ -6,7 +6,7 @@
  */
 
 /*
-    Copyright 2011 Thibaut Paumard
+    Copyright 2011-2015 Thibaut Paumard
 
     This file is part of Gyoto.
 
@@ -33,12 +33,20 @@ namespace Gyoto{
 
 #include <GyotoDefs.h>
 #include <GyotoSmartPointer.h>
+#include <GyotoObject.h>
 #include <GyotoAstrobj.h>
 #include <GyotoMetric.h>
 #include <GyotoScreen.h>
 #include <GyotoPhoton.h>
 #include <GyotoConverters.h>
 
+#ifdef HAVE_MPI
+#include "GyotoFactory.h"
+#include <boost/mpi/environment.hpp>
+#include <boost/mpi/communicator.hpp>
+#endif
+
+
 /**
  * \class Gyoto::Scenery
  * \brief Ray-tracing scene
@@ -151,7 +159,10 @@ namespace Gyoto{
  * </Scenery>
  * \endcode
  */
-class Gyoto::Scenery : protected Gyoto::SmartPointee {
+class Gyoto::Scenery
+: public Gyoto::SmartPointee,
+  public Gyoto::Object
+{
   friend class Gyoto::SmartPointer<Gyoto::Scenery>;
   
   
@@ -193,6 +204,8 @@ class Gyoto::Scenery : protected Gyoto::SmartPointee {
    */
   size_t nthreads_; ///< Number of parallel threads to use in rayTrace()
 
+  int nprocesses_; ///< Number of parallel processes to use in rayTrace()
+
 # ifdef HAVE_UDUNITS
   /// See Astrobj::Properties::intensity_converter_
   Gyoto::SmartPointer<Gyoto::Units::Converter> intensity_converter_;
@@ -202,11 +215,48 @@ class Gyoto::Scenery : protected Gyoto::SmartPointee {
   Gyoto::SmartPointer<Gyoto::Units::Converter> binspectrum_converter_;
 # endif
 
-  size_t maxiter_ ; ///< Maximum number of iterations when integrating
+ public:
+# ifdef HAVE_MPI
+  /// Team of processes for MPI
+  /**
+   * Rank 0 is the manager, other ranks are workers, instances of the
+   * gyoto-mpi-worker executable.
+   */
+  boost::mpi::communicator * mpi_team_;
+# endif
+  /// True in instance of gyoto-mpi-worker, otherwise false.
+  static bool am_worker;
+
+  /// Spawn gyoto-mpi-worker processes
+  /**
+   * Also sets nprocesses_. If the right number of workers is already
+   * running, does nothing.  Also does nothing if MPI_Init() has not
+   * been called yet.
+   */
+  void mpiSpawn(int nbchildren);
+
+  /// Terminate gyoto-mpi-worker-processes
+  void mpiTerminate ();
+
+  /// Send a copy of self to the mpi workers
+  /**
+   * Always call mpiClone() before ray-tracing if workers are running.
+   */
+  void mpiClone();
+
+  /// Tags that may be sent to communicate with workers using MPI_Send()
+  enum mpi_tag {give_task, read_scenery, terminate,
+		raytrace, raytrace_done, ready,
+		impactcoords, noimpactcoords};
+
+  /// Send a tag to workers
+  void mpiTask(mpi_tag &tag);
 
   // Constructors - Destructor
   // -------------------------
  public:
+  GYOTO_OBJECT;
+  GYOTO_WORLDLINE;
   Scenery(); ///< Set everything to defaults
   Scenery (const Scenery& o); ///< Copy constructor
   Scenery * clone() const; ///< Cloner
@@ -244,12 +294,19 @@ class Gyoto::Scenery : protected Gyoto::SmartPointee {
 
 
   SmartPointer<Photon> clonePhoton() const; ///< Clone the internal Photon
+  SmartPointer<Photon> clonePhoton(size_t i, size_t j); ///< Clone the internal Photon
+  SmartPointer<Photon> clonePhoton(double a, double d); ///< Clone the internal Photon
+  void updatePhoton(); ///< Update values in cached Photon
 
   double delta() const ; ///< Get default step in geometrical units
   double delta(const std::string &unit) const ;  ///< Get default step in specified units
   void delta(double); ///< set default step in geometrical units
   void delta(double, const std::string &unit);   ///< set default step in specified units
 
+  void initCoord(std::vector<double> c);
+  std::vector<double> initCoord() const;
+
+
   /// Set Scenery::quantities_
   /**
    * \param quant Bitwise OR of desired quantities, e.g. \code GYOTO_QUANTITY_SPECTRUM | GYOTO_QUANTITY_MIN_DISTANCE \endcode
@@ -261,19 +318,19 @@ class Gyoto::Scenery : protected Gyoto::SmartPointee {
    * \param squant Coma-separated list of quantities, e.g. "Spectrum
    * MinDistance". The order is not relevant.
    */
-  void setRequestedQuantities(std::string squant) ;
+  void requestedQuantitiesString(std::string const &squant) ;
 
   /// Get Scenery::quantities_
   Quantity_t getRequestedQuantities() const ;
 
   /// Get a string representation of Scenery::quantities_
-  std::string getRequestedQuantitiesString() const ;
+  std::string requestedQuantitiesString() const ;
 
   /// Get number of requested quantities of scalar nature
   /**
    * This is all quantities except Spectrum, BinSpectrum and ImpactCoords.
    */
-  size_t getScalarQuantitiesCount() const ;
+  size_t getScalarQuantitiesCount(Quantity_t *q=NULL) const ;
 
   /// Get ph_.tmin_
   double tMin() const ;
@@ -326,6 +383,9 @@ class Gyoto::Scenery : protected Gyoto::SmartPointee {
   void nThreads(size_t); ///< Set nthreads_;
   size_t nThreads() const ; ///< Get nthreads_;
 
+  void nProcesses(size_t); ///< Set nprocesses_;
+  size_t nProcesses() const ; ///< Get nprocesses_;
+
   /// Set Scenery::intensity_converter_
   void intensityConverter(std::string unit);
   /// Set Scenery::spectrum_converter_
@@ -342,10 +402,9 @@ class Gyoto::Scenery : protected Gyoto::SmartPointee {
 
   // Worker:
  public:
-  /// Perform ray-tracing for a square area on Screen
+  /// Perform ray-tracing
   /**
-   * For each Scenery::screen_ pixel in the square area limited by
-   * imin, imax, jmin and jmax, launch a Photon back in time to
+   * For each directions specified, launch a Photon back in time to
    * compute the various quantities.
    *
    * At this time, the computed quantities depend on on the pointers
@@ -360,7 +419,11 @@ class Gyoto::Scenery : protected Gyoto::SmartPointee {
    * the various pointers in *data must be NULL or point to the first
    * cell in an array of size at least Screen::npix_ squared.
    *
-   * If Scenery::nthreads_ is &ge;2 and Gyoto has been compiled with
+   * If MPI support is built-in, MPI_Init() has been called, and
+   * nprocesses_ is &ge;1, then rayTrace() will use several processes,
+   * launching them using mpiSpawn() if necessary.
+   *
+   * Else, if Scenery::nthreads_ is &ge;2 and Gyoto has been compiled with
    * pthreads support, rayTrace() will use Scenery::nthreads_ threads
    * and launch photons in parallel. This works only if the
    * Astrobj::Generic::clone() and Metric::Generic::clone() methods
@@ -368,9 +431,14 @@ class Gyoto::Scenery : protected Gyoto::SmartPointee {
    * metric kind, and if they are both thread-safe. At the moment,
    * unfortunately, Lorene metrics are known to not be thread-safe.
    *
-   * \param[in] imin, imax, jmin, jmax First and last rows and columns in
-   * Scenery::screen_ to compute
-
+   * \param[in] ij Screen::Coord2dSet specification of rays to trace. e.g.:
+   *
+   * \code
+   * Screen::Range irange(imin, imax, di); 
+   * Screen::Range jrange(jmin, jmax, dj);
+   * Screen::Grid ij(irange, jrange); 
+   * \endcode
+   *
    * \param[in, out] data Pointer to a preallocated
    * Astrobj::Properties instance which sets which quantities must be
    * computed and where to store the output.
@@ -384,9 +452,14 @@ class Gyoto::Scenery : protected Gyoto::SmartPointee {
    * distributions are not. impactcoords can be computed using the
    * ImpactCoords quantity.
    */
-  void rayTrace(size_t imin, size_t imax, size_t jmin, size_t jmax,
-		Astrobj::Properties* data, double * impactcoords = NULL);
-
+  void rayTrace(
+#ifdef GYOTO_SWIGIMPORTED
+		Coord2dSet & ij,
+#else
+		Screen::Coord2dSet & ij,
+#endif
+		Astrobj::Properties *data,
+		double * impactcoords=NULL);
 
   /// Ray-trace a single pixel in Scenery::screen_
   /**
@@ -399,14 +472,23 @@ class Gyoto::Scenery : protected Gyoto::SmartPointee {
   void operator() (size_t i, size_t j, Astrobj::Properties *data,
 		   double * impactcoords = NULL, Photon * ph = NULL);
 
+  /// Ray-trace single direction
+  /**
+   * Almost identical to rayTrace(), but for a single direction.
+   *
+   * If ph is passed, it is assumed to have been properly initialized
+   * (with the right metric and astrobj etc.) already. Else, use
+   * &Scenery::ph_.
+   */
+  void operator() (double alpha, double delta, Astrobj::Properties *data,
+		   Photon * ph = NULL);
+
 #ifdef GYOTO_USE_XERCES
  public:
-  /// Fill XML section
-  /**
-   * Akin to Astrobj::Generic::fillElement or
-   * Metric::Generic::fillElement for instance.
-   */
-  void fillElement(FactoryMessenger *fmp);
+  // Override fillProperty() to issue InitCoord only if it was set
+  void fillProperty(FactoryMessenger *fmp, Property const &p) const ;
+  // Override fillElement to fill metric, screen and astrobj first
+  void fillElement(FactoryMessenger *fmp) const;
   /// Instanciate Scenery from an XML description.
   static SmartPointer<Scenery> Subcontractor(Gyoto::FactoryMessenger*);
 
