@@ -28,6 +28,7 @@
 #define __GyotoSpectrum_H_ 
 
 #include "GyotoRegister.h"
+#include "GyotoObject.h"
 
 namespace Gyoto{
   namespace Register { class Entry; }
@@ -62,7 +63,7 @@ namespace Gyoto{
       (FactoryMessenger* fmp) {
       SmartPointer<T> sp = new T();
 #ifdef GYOTO_USE_XERCES
-      sp -> setParameters(fmp);
+      if (fmp) sp -> setParameters(fmp);
 #endif
       return sp;
     }
@@ -122,14 +123,18 @@ namespace Gyoto{
  *  Light emitted by e.g. a Star
  *
  */
-class Gyoto::Spectrum::Generic : protected Gyoto::SmartPointee {
+class Gyoto::Spectrum::Generic
+: public Gyoto::SmartPointee,
+  public Gyoto::Object
+{
   friend class Gyoto::SmartPointer<Gyoto::Spectrum::Generic>;
  protected:
-  std::string kind_; ///< e.g. constants, blackbody...
 
  public:
+  GYOTO_OBJECT;
+
   Generic(const std::string kind); ///< Set kind in constructor
-  //  Spectrum::Generic(const Spectrum::Generic &); ///< Copy constructor. Default is fine.
+  Generic(const Spectrum::Generic &);
   virtual Generic * clone() const; ///< Cloner
 
   virtual ~Generic() ; ///< Destructor: does nothing.
@@ -172,43 +177,6 @@ class Gyoto::Spectrum::Generic : protected Gyoto::SmartPointee {
   virtual double integrate(double nu1, double nu2,
 			   const Spectrum::Generic * opacity, double ds) ;
 
-  virtual void setParameter(std::string name,
-			    std::string content,
-			    std::string unit) ;
-  ///< Set any parameter by its name
-
-#ifdef GYOTO_USE_XERCES
-  /**
-   * Spectrum implementations should impement fillElement to save their
-   * parameters to XML and call the generic implementation to save
-   * generic parts.
-   */
-
-  virtual void fillElement(FactoryMessenger *fmp) const ;
-                                             ///< called from Factory
-
-  /**
-   * The Subcontractor_t function for each Spectrum kind should look
-   * somewhat like this:
-   * \code
-   * SmartPointer<Spectrum::Generic>
-   * Gyoto::Spectrum::MyKind::Subcontractor(FactoryMessenger* fmp) {
-   *   SmartPointer<MyKind> sp = new MyKind();
-   *   sp -> setParameters(fmp);
-   *   return sp;
-   * }
-   * \endcode
-   *
-   * Each spectrum kind should implement setParameter(string name,
-   * string content) to interpret the individual XML
-   * elements. setParameters() can be overloaded in case the specific
-   * Spectrum class needs low level access to the FactoryMessenger (see
-   * Astrobj::UniformSphere::setParameters()).
-   */
-  virtual void setParameters(FactoryMessenger *fmp);
-  ///< Main loop in Subcontractor_t function
-
-#endif
 };
 
 
