@@ -4,7 +4,7 @@
  */
 
 /*
-    Copyright 2011 Frederic Vincent, Thibaut Paumard
+    Copyright 2011-2015 Frederic Vincent, Thibaut Paumard
 
     This file is part of Gyoto.
 
@@ -31,7 +31,7 @@
 
 #include <GyotoDefs.h>
 
-#ifdef HAVE_BOOST
+#ifdef GYOTO_HAVE_BOOST_INTEGRATORS
 # include <functional>
 # include <array>
 # include <boost/numeric/odeint/stepper/controlled_step_result.hpp>
@@ -47,6 +47,127 @@ namespace Gyoto {
 #include <GyotoScreen.h>
 #include <GyotoHooks.h>
 
+/// Define the bunch of Properties that make up the Worldline interface
+/**
+ * This macro, which is called automatically by
+ * GYOTO_WORLDLINE_PROPERTY_END(c, a), must be inserted in the
+ * definition of the Property list for any class derived from
+ * Worldline.
+ */
+#define GYOTO_WORLDLINE_PROPERTIES(c)					\
+  GYOTO_PROPERTY_BOOL(c, HighOrderImages, PrimaryOnly, _secondary)	\
+  GYOTO_PROPERTY_DOUBLE(c, RelTol, _relTol)				\
+  GYOTO_PROPERTY_DOUBLE(c, AbsTol, _absTol)				\
+  GYOTO_PROPERTY_DOUBLE(c, DeltaMaxOverR, _deltaMaxOverR)		\
+  GYOTO_PROPERTY_DOUBLE(c, DeltaMax, _deltaMax)				\
+  GYOTO_PROPERTY_DOUBLE(c, DeltaMin, _deltaMin)				\
+  GYOTO_PROPERTY_STRING(c, Integrator, _integrator)			\
+  GYOTO_PROPERTY_SIZE_T(c, MaxIter, _maxiter)				\
+  GYOTO_PROPERTY_BOOL(c, Adaptive, NonAdaptive, _adaptive)		\
+  GYOTO_PROPERTY_DOUBLE_UNIT(c, MinimumTime, _tMin)			\
+  GYOTO_PROPERTY_DOUBLE_UNIT(c, Delta, _delta)				\
+  GYOTO_PROPERTY_VECTOR_DOUBLE(c, InitCoord, _initCoord)		\
+  GYOTO_PROPERTY_METRIC(c, Metric, _metric)
+
+/// Define the wrapper accessors used in GYOTO_WORLDLINE_PROPERTIES(class)
+/**
+   This macro, which is called automatically by
+   GYOTO_WORLDLINE_PROPERTY_END(c, a), must be called once with the
+   definition of the methods (.C file) of any class that derives from
+   Worldline. The corresponding macro GYOTO_WORLDLINE must be called
+   in the corresponding class declaration (.h file).
+
+   This is made necessary by how multiple inheritence works: directly
+   using the accessors in the Worldline API leads to segfault at
+   runtime (unless too much extra care is taken) and may go unnoticed.
+
+   These accessors must be declared in the class declaration using the
+   GYOTO_WORLDLINE macro.
+*/
+#define GYOTO_WORLDLINE_ACCESSORS(c)					\
+  void c::_secondary(bool s) {secondary(s);}				\
+  bool c::_secondary() const {return secondary();}			\
+  void c::_adaptive(bool s) {adaptive(s);}				\
+  bool c::_adaptive() const {return adaptive();}			\
+  void c::_relTol(double f){relTol(f);}					\
+  double c::_relTol()const{return relTol();}				\
+  void c::_absTol(double f){absTol(f);}					\
+  double c::_absTol()const{return absTol();}				\
+  void c::_deltaMin(double f){deltaMin(f);}				\
+  double c::_deltaMin()const{return deltaMin();}			\
+  void c::_deltaMax(double f){deltaMax(f);}				\
+  double c::_deltaMax()const{return deltaMax();}			\
+  void c::_deltaMaxOverR(double f){deltaMaxOverR(f);}			\
+  double c::_deltaMaxOverR()const{return deltaMaxOverR();}		\
+  void c::_delta(double f){delta(f);}					\
+  double c::_delta()const{return delta();}				\
+  void c::_delta(double f, std::string const &u){delta(f, u);}		\
+  double c::_delta(std::string const &u)const{return delta(u);}		\
+  void c::_tMin(double f){tMin(f);}					\
+  double c::_tMin()const{return tMin();}				\
+  void c::_tMin(double f, std::string const &u){tMin(f, u);}		\
+  double c::_tMin(std::string const &u)const{return tMin(u);}		\
+  void c::_maxiter(size_t f){maxiter(f);}				\
+  size_t c::_maxiter()const{return maxiter();}				\
+  void c::_integrator(std::string const &f){integrator(f);}		\
+  std::string c::_integrator() const {return integrator();}		\
+  std::vector<double> c::_initCoord()const{return initCoord();}		\
+  void c::_initCoord(std::vector<double> const &f){initCoord(f);}	\
+  void c::_metric(SmartPointer<Metric::Generic>f){metric(f);}		\
+  SmartPointer<Metric::Generic> c::_metric() const{return metric();}
+
+/// Drop-in replacement for GYOTO_PROPERTY_END(), which adds the Worldline interface
+/**
+ * This macro replaces GYOTO_PROPERTY_END(c, a) for classes that
+ * derive from Worldline. It calls GYOTO_WORLDLINE_PROPERTIES(a) and
+ * GYOTO_WORLDLINE_ACCESSORS(c). If this macro is used,
+ * GYOTO_WORLDLINE must be called in the class declaration (.h file).
+ */
+#define GYOTO_WORLDLINE_PROPERTY_END(c, a) \
+  GYOTO_WORLDLINE_PROPERTIES(c)		   \
+  GYOTO_PROPERTY_END(c, a)		   \
+  GYOTO_WORLDLINE_ACCESSORS(c)
+
+/// Declare the Worldline interface wrappers
+/**
+   This macro must be called in the class declaration (.h file), in a
+   public section. Its sibling GYOTO_WORLDLINE_ACCESSORS(c) must be
+   called with the class method definition (.C file). Note that
+   GYOTO_WORLDLINE_PROPERTY_END(c, a) calls
+   GYOTO_WORLDLINE_ACCESSORS(c).
+*/
+#define GYOTO_WORLDLINE					\
+  void _delta(const double delta);			\
+  void _delta(double, const std::string &unit);		\
+  double _delta() const ;				\
+  double _delta(const std::string &unit) const ;	\
+  void _tMin(const double tmin);			\
+  void _tMin(double, const std::string &unit);		\
+  double _tMin() const ;				\
+  double _tMin(const std::string &unit) const ;		\
+  void _adaptive (bool mode) ;				\
+  bool _adaptive () const ;				\
+  void _secondary (bool sec) ;				\
+  bool _secondary () const ;				\
+  void _maxiter (size_t miter) ;			\
+  size_t _maxiter () const ;				\
+  void _integrator(std::string const & type);		\
+  std::string _integrator() const ;			\
+  double _deltaMin() const;				\
+  void _deltaMin(double h1);				\
+  void _absTol(double);					\
+  double _absTol()const;				\
+  void _relTol(double);					\
+  double _relTol()const;				\
+  void _deltaMax(double h1);				\
+  double _deltaMax()const;				\
+  double _deltaMaxOverR() const;			\
+  void _deltaMaxOverR(double t);			\
+  std::vector<double> _initCoord()const;		\
+  void _initCoord(std::vector<double> const&f);		\
+  void _metric(SmartPointer<Metric::Generic>);		\
+  SmartPointer<Metric::Generic> _metric() const;
+
 /**
  * \class Gyoto::Worldline
  * \brief  Timelike or null geodesics
@@ -63,19 +184,24 @@ namespace Gyoto {
  * extended as needed using xExpand(). These coordinates can be
  * retrieved using get_t(), get_xyz(), getCartesian(), getCoord() etc.
  *
- * Supported XML parameters:
- *  - InitialCoordinate or InitCoord: 8-element vector yielding the initial
- *    4-position and 4-velocity;
- *  - Only for massive particle (Gyoto::Astrobj::Star): Position
- *    (yielding initial 4-position) and Velocity (yielding initial
- *    3-velocity);
- *  - Delta: integration step, initial in case or adaptive step;
- *  - Adaptive or NonAdaptive: sets whether integration step should be
- *    adaptive; default: Adaptive.;
- *  - MaxIter: maximum number of iterations for the integration;
- *    default: 100000.
- *  - DeltaMin, DeltaMax, DeltaMaxOverR, AbsTol, RelTol: tuning
- *    parameters used by some of the integrators.
+ * Worldline does not derive from Object, and does not instanciate a
+ * Property list. This is because this would lead to multiple
+ * inheritance of the Object base in derived classes. Instead,
+ * #GyotoWorldline.h provides a few macros that can be used to include
+ * the Worldline properties in a derived classe's Property list:
+ *   - #GYOTO_WORLDLINE is to be used in a public section of the
+ *     derived class declaration (.h file); it declares wrappers
+ *     around the Worldline property accessors;
+ *   - #GYOTO_WORLDLINE_ACCESSORS is to be used with the class
+ *     definition (.C file; it defines the accessors declared by
+ *     #GYOTO_WORLDLINE;
+ *   - #GYOTO_WORLDLINE_PROPERTIES declares the Properties that use
+ *     these accessors. It must be used like
+ *     e.g. #GYOTO_PROPERTY_DOUBLE, between #GYOTO_PROPERTY_START andf
+ *     #GYOTO_PROPERTY_END.
+ *   - Finally, #GYOTO_WORLDLINE_PROPERTY_END is a drop-in replacement
+ *     for #GYOTO_PROPERTY_END that calls #GYOTO_WORLDLINE_PROPERTIES
+ *     and #GYOTO_WORLDLINE_ACCESSORS.
  * 
  */
 class Gyoto::Worldline
@@ -216,6 +342,10 @@ class Gyoto::Worldline
   virtual double getMass() const = 0; ///< Get mass of particule.
   void   metric(SmartPointer<Metric::Generic>); ///< Set metric Smartpointer
   SmartPointer<Metric::Generic> metric() const; ///< Get metric
+
+  void initCoord(std::vector<double> const&); 
+  std::vector<double> initCoord() const; 
+
   virtual void   setInitCoord(const double coord[8], int dir = 0); ///< Set Initial coordinate
 
   /**
@@ -241,12 +371,12 @@ class Gyoto::Worldline
    *
    * Initialize #state_ to use the required integrator.
    *
-   * \param[in] type Either "Legacy" or (if HAVE_BOOST) one of
+   * \param[in] type Either "Legacy" or (if GYOTO_HAVE_BOOST_INTEGRATORS) one of
    *                 "runge_kutta_cash_karp54",
    *                 "runge_kutta_fehlberg78", "runge_kutta_dopri5",
    *                 "runge_kutta_cash_karp54_classic"
    */
-  void integrator(std::string type);
+  void integrator(std::string const & type);
 
   /**
    * \brief Describe the integrator used by #state_
@@ -334,7 +464,6 @@ class Gyoto::Worldline
   // ---------------------
  public:
   /// Assignment to another Worldline
-  void operator=(const Worldline&) ;        
   void delta(const double delta); ///< Set #delta_
   void delta(double, const std::string &unit);   ///< Set #delta_ in specified units
   double delta() const ; ///< Get #delta_
@@ -383,47 +512,7 @@ class Gyoto::Worldline
   virtual void xStore(size_t ind, double coord[8]) ; ///< Store coord at index ind
   virtual void xFill(double tlim) ; ///< Fill x0, x1... by integrating the Worldline from previously set inittial condition to time tlim
 
-  /**
-   * \brief Set parameter by name
-   *
-   * Assume MyKind is a subclass of Worldline which has two
-   * members (a string StringMember and a double DoubleMember):
-   * \code
-   * int MyKind::setParameter(std::string name,
-   *                          std::string content,
-   *                          std::string unit) {
-   *   if      (name=="StringMember") setStringMember(content);
-   *   else if (name=="DoubleMember") setDoubleMember(atof(content.c_str()),
-   *                                                  unit);
-   *   else return Worldline::setParameter(name, content, unit);
-   *   return 0;
-   * }
-   * \endcode
-   *
-   * \param name XML name of the parameter
-   * \param content string representation of the value
-   * \param unit string representation of the unit
-   * \return 0 if this parameter is known, 1 if it is not.
-   */
-  virtual int setParameter(std::string name,
-			   std::string content,
-			   std::string unit) ;
 
-#ifdef GYOTO_USE_XERCES
-  /**
-   * \brief Process XML entity
-   * Uses wait_pos_ and init_vel_ to make sure setVelocity() is called
-   * after setPosition().
-   */
-  virtual void setParameters(FactoryMessenger *fmp) ;
-  /**
-   * Derived classes implementations should implement fillElement to save their
-   * parameters to XML and call the generic implementation to save
-   * generic parts such as adaptive_: Worldline::fillElement(fmp).
-   */
-  virtual void fillElement(FactoryMessenger *fmp) const ;
-                                             ///< XML output
-#endif
 
   // Accessors
   // ---------
@@ -532,9 +621,6 @@ class Gyoto::Worldline
   void save_txyz(char* const filename, double const t1, double const  mass_sun,
 		 double const distance_kpc, std::string const unit, SmartPointer<Screen> sc = NULL);///< Save, converted
 
-  /// Display
-  friend std::ostream& operator<<(std::ostream& , const Worldline& ) ;
-  
  protected:
   virtual void tell(Gyoto::Hook::Teller*);
 
@@ -542,7 +628,7 @@ class Gyoto::Worldline
   public:
     class Generic;
     class Legacy;
-#ifdef HAVE_BOOST
+#ifdef GYOTO_HAVE_BOOST_INTEGRATORS
     class Boost;
 #endif
   };
@@ -554,12 +640,12 @@ class Gyoto::Worldline
   SmartPointer<Worldline::IntegState::Generic> state_;
 };
 
-
+#ifndef GYOTO_SWIGIMPORTED
 /**
  * \class Gyoto::Worldline::IntegState::Generic
  * \brief Current state of a geodesic integration
  */
-class Gyoto::Worldline::IntegState::Generic : SmartPointee {
+class Gyoto::Worldline::IntegState::Generic : public SmartPointee {
   friend class Gyoto::SmartPointer<Gyoto::Worldline::IntegState::Generic>;
  protected:
   /// Worldline that we are integrating.
@@ -636,6 +722,19 @@ class Gyoto::Worldline::IntegState::Generic : SmartPointee {
    */
   virtual int nextStep(double *coord, double h1max=GYOTO_DEFAULT_DELTA_MAX)=0;
 
+  /// Make one step of exactly this size.
+  /**
+   * doStep() is meant to refine a computation made using
+   * nextStep(). In particular, there is no checking for norm
+   * conservation.
+   *
+   * \param[in] coordin current position-velocity;
+   * \param[in] step  exact step to use.
+   * \param[out] coordout next position-velocity;
+   */
+  virtual void doStep(double const coordin[8], 
+		      double step,
+		      double coordout[8])=0;
 };
 
 /**
@@ -664,17 +763,16 @@ class Gyoto::Worldline::IntegState::Legacy : public Generic {
   void init(Worldline * line, const double *coord, const double delta);
   virtual std::string kind();
 
-  /// Make one step.
-  /**
-   * \param[out] coord Next position-velocity;
-   * \param[in] h1max maximum step in case of adaptive integration
-   */
   virtual int nextStep(double *coord, double h1max=1e6);
 
+  virtual void doStep(double const coordin[8], 
+		      double step,
+		      double coordout[8]);
+
   virtual ~Legacy();
 };
 
-#ifdef HAVE_BOOST
+#ifdef GYOTO_HAVE_BOOST_INTEGRATORS
 /**
  * \class Gyoto::Worldline::IntegState::Boost
  * \brief Boost integrator
@@ -726,9 +824,12 @@ class Gyoto::Worldline::IntegState::Boost : public Generic {
   virtual void init();
   virtual void init(Worldline * line, const double *coord, const double delta);
   virtual int nextStep(double *coord, double h1max=1e6);
+  virtual void doStep(double const coordin[8], 
+		      double step,
+		      double coordout[8]);
   virtual std::string kind();
   
 };
-#endif /// HAVE_BOOST
-
+#endif /// GYOTO_HAVE_BOOST_INTEGRATORS
+#endif /// GYOTO_SWIGIMPORTED
 #endif
