@@ -273,7 +273,8 @@ void ygyoto_Spectrometer_generic_eval
       unit="";								\
     }									\
     if (yarg_nil(argc-2)) {						\
-      if (prop_pushed++) y_error("Can push only one return value");	\
+      if (prop_pushed) y_error("Can push only one return value");	\
+      prop_pushed=true;							\
       yarg_drop(1);							\
       ypush_property(*YGYOTO_CAT(OBJ, _), *prop, pname, unit);	\
     }									\
@@ -304,7 +305,7 @@ void ygyoto_Spectrometer_generic_eval
   char const * pmsg="Cannot use positional argument more than once";	\
   char * unit=NULL;							\
   /* make sure we use the variables at least once */			\
-  if (OBJ && rmsg && pmsg && unit) {}
+  if (OBJ && rmsg && pmsg && unit && k && *paUsed) {}
 
 
 #define YGYOTO_WORKER_GETSET_VECTOR(MEMBER, N)			  \
